use std::fs;

fn parse_steps(lines: std::str::Lines) -> Vec<Vec<Vec<u32>>> {
    let mut steps: Vec<Vec<Vec<u32>>> = Vec::new();
    let mut step: Vec<Vec<u32>> = Vec::new();
    for line in lines {
        if line.contains("-to-") {
            step = Vec::new();
            steps.push(step);
        } else if line != "" {
            let numbers: Vec<&str> = line.split(" ").collect();
            let numbers: Vec<u32> = numbers
                .iter()
                .map(|string| {
                    let number: u32 = string.parse().unwrap();
                    number
                })
                .collect();
            let n = steps.len();
            steps[n - 1].push(numbers);
        }
    }
    steps
}

fn position_min(vector: Vec<u32>) -> Option<usize> {
    if vector.len() == 0 {
        return None;
    }
    let mut min_index = 0;
    let mut min = vector[0];
    for (i, value) in vector.iter().enumerate() {
        if *value < min {
            min_index = i;
            min = *value;
        }
    }
    Some(min_index)
}

fn seed_to_location(seed: u32, steps: &Vec<Vec<Vec<u32>>>) -> u32 {
    let mut current: u32 = seed;
    println!("{:?}", current);
    for step in steps {
        let mut updated = false;
        for range in step {
            let destination = range[0];
            let source = range[1];
            let size = range[2];
            if current >= source && current <= source + size {
                if !updated {
                    current = destination + current - source;
                } else {
                    println!("Already updated?");
                }
            }
        }
        println!("{:?}", current);
    }
    println!("\n");
    current
}

fn main() {
    let filepath: &str = "../../data/sample.txt";

    let contents = fs::read_to_string(filepath).expect("Should have been hable to read the file");

    let mut lines = contents.lines();
    let seeds: Vec<&str> = lines
        .next()
        .unwrap()
        .split(": ")
        .nth(1)
        .expect("Should have a seed")
        .split(" ")
        .collect();
    let steps = parse_steps(lines);
    let seeds: Vec<u32> = seeds
        .iter()
        .map(|string| {
            let number: u32 = string.trim().parse().unwrap();
            number
        })
        .collect();
    let locations: Vec<u32> = seeds
        .iter()
        .map(|seed| seed_to_location(*seed, &steps))
        .collect();
    let min_location = locations.iter().min();
    // let min_location_index = position_min(locations).unwrap();
    println!("{:?}", min_location);
}
